package app.cash.sqldelight.intellij.actions

import app.cash.sqldelight.core.lang.util.findChildOfType
import app.cash.sqldelight.core.psi.SqlDelightStmtList
import com.alecstrong.sql.psi.core.psi.SqlCreateTableStmt
import com.intellij.codeInsight.CodeInsightActionHandler
import com.intellij.codeInsight.template.TemplateActionContext
import com.intellij.codeInsight.template.TemplateManager
import com.intellij.codeInsight.template.impl.TemplateManagerImpl
import com.intellij.openapi.editor.Editor
import com.intellij.openapi.project.Project
import com.intellij.psi.PsiFile
import com.intellij.psi.util.parentOfType

internal class GenerateInsertIntoQueryAction : BaseGenerateAction(InsertIntoHandler()) {
  class InsertIntoHandler : CodeInsightActionHandler {
    override fun invoke(project: Project, editor: Editor, file: PsiFile) {
      val caretOffset = editor.caretModel.offset
      val createTableStmt = file.findElementAt(caretOffset)?.parentOfType<SqlCreateTableStmt>() ?: return

      val tableName = createTableStmt.tableName.name
      val template = TemplateManagerImpl.listApplicableTemplates(TemplateActionContext.create(file, null, caretOffset, caretOffset, false))
        .first { it.key == "ins" }

      val stmtList = file.findChildOfType<SqlDelightStmtList>() ?: return
      insertNewLineAndCleanup(editor, stmtList)
      TemplateManager.getInstance(project).startTemplate(
        editor, template, false, mapOf("table" to tableName), null
      )
    }
  }
}
